/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: ILogAnalyzer.java,v 1.7 2005/02/16 22:19:53 qiyanli Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/

package org.eclipse.hyades.analysis.engine;

/**
 *	This interface is required to be implemented by pluggable log analyzers
 */
public interface ILogAnalyzer {

	/**
	 * Analyzes an object or a collection of objects.
	 * @param aObject the object to be analyzed.
	 * @param monitor the progress monitor to use, or
	 *    <code>null</code> if monitoring is not required.
	 * @return the result of the analysis process. 
	 */	
	public String analyze(Object aObject, IAnalysisMonitor monitor);
	
	/**
	 * Returns an error message if the analyze process fails for some reason.
	 */		
	public String errorMsg();
	
	
	/**
	 * Loads the symptom databases used for log analysis.	 
	 */
	public void loadDatabase();
	
	/**
	 * Unloads the symptom databases used for log analysis.	 
	 */
	public void unloadDatabase();
	
}
