/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: RecorderRegistryInfo.java,v 1.16 2005/04/29 13:50:50 mddunn Exp $
 * 
 * Contributors: 
 * IBM Corporation - initial API and implementation
 **********************************************************************/

/*
 * Created on Mar 4, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package org.eclipse.hyades.internal.execution.recorder.ui.wizards;

import java.io.File;
import java.io.FileInputStream;

import java.io.IOException;
import java.io.InputStreamReader;
import java.util.StringTokenizer;

/**
 * @author mdunn
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class RecorderRegistryInfo {
	
	private String proxyType = "";
	private String proxyAddr = "";
	private String proxyPort = "";
	private boolean isProxyEnabled = false;
	private boolean badProxySettings = false;
	private int proxyError = 0;
	// defect 82974 1/18/2005
	private boolean isAutoConfigURLEnabled = false;
	private boolean isProxyOverrideEnabled = false;
	
	public final int PROXY_SETTINGS_OK = 0;
	public final int SOCKS_SET = -1;
	public final int HTTP_PROXY_ONLY_SET = -2;
	public final int SECURE_PROXY_ONLY_SET = -3;
	public final int HTTP_SECURE_MISMATCH = -4;
	public final int HTTP_SECURE_LOCALHOST = -5;
	// defect 82974 1/18/2005 next 2 lines
	public final int HTTP_PROXY_OVERRIDE = -6;
	public final int AUTO_CONFIG_URL = -7;
	// bugzilla 79219 mdd
	private String sslProxyAddr = "";
	private String sslProxyPort = "";
	private String secureStringGlobal = "";
	private boolean isSSLProxyEnabled = false;
	// new stuff bugzilla 89968 mdd
	private String proxyOverrideString = "";
	private String autoConfigURLString = "";
	
	
	public RecorderRegistryInfo() {
		StringBuffer inputFileContents = getRegistryInfo();
		boolean isEnabled = checkProxyEnabled(inputFileContents);
		setProxyEnabled(isEnabled);
		if (! isEnabled ) {
			setProxyType("socks");
		}
		else {	
			// must do it new way for hyades 1.3 mdd
			// make sure it is http proxy, grab the address and port
			// then set ProxyServer http=localhost:1080
			String proxyInfo = getProxyInfo(inputFileContents);
			if (isProxyEnabled && isProxyOverrideEnabled) {
				setBadProxySettings(true);
				setProxyError(HTTP_PROXY_OVERRIDE);
			}
			if (proxyInfo.length()!= 0) {
				String[] proxySplit = proxyInfo.split(":");
				String myProxyAddr = proxySplit[0];
				String myProxyPort = proxySplit[1];
				setProxyAddr(myProxyAddr);	
				setProxyPort(myProxyPort);
				setProxyType("http");
				// defect 79219
				if (secureStringGlobal.length()!= 0) {
					String[] sslSplit = secureStringGlobal.split(":");
					String mysslProxyAddr = sslSplit[0];
					String mysslProxyPort = sslSplit[1];
					setSslProxyAddr(mysslProxyAddr);	
					setSslProxyPort(mysslProxyPort);
				}
			}
			else {
				setProxyType("socks");
			}
		}
	}

	private StringBuffer getRegistryInfo()
	{
		String originalSettingsFileName = "newSettingsFile.tmp";
		String getOriginalInfoCommandLine = "regedit /e "+originalSettingsFileName+" \"HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings\"";//$NON-NLS-1$//$NON-NLS-2$
		InputStreamReader inputReader=null;
		File file = null;
		StringBuffer inputFileContents = new StringBuffer();
		try
		{
			try {
			Process registryChange = Runtime.getRuntime().exec(getOriginalInfoCommandLine,null);
			registryChange.waitFor();
			}
			catch (InterruptedException e) {}
			file = new File(originalSettingsFileName);
			inputReader= new InputStreamReader(new FileInputStream(file),"UTF-16");//$NON-NLS-1$
			inputFileContents = new StringBuffer();
			char[] inputBuf = new char[1000];
			int bytesRead = inputReader.read(inputBuf);
								
			while(bytesRead!=-1)
			{				
				String buf = new String(inputBuf);				
				inputFileContents.append(inputBuf,0,bytesRead);
				bytesRead = inputReader.read(inputBuf);
			}			
		}
		catch (IOException e)
		{
			e.printStackTrace();
		} 

		finally
		{
			if(inputReader!=null)
				try
				{
					inputReader.close();
				}
				catch (IOException e1)
				{
					e1.printStackTrace();
				}
		}
		// Clean up
		if (file != null) {
			if (file.exists())
				file.delete();
		}
		return inputFileContents;
	}
	private boolean checkProxyEnabled(StringBuffer inputBuffer) 
	{
			boolean comparesOK = false;
			StringTokenizer fileTokenizer = new StringTokenizer(inputBuffer.toString(),"\r\n");//$NON-NLS-1$
			while(fileTokenizer.hasMoreTokens())
			{
				String line=fileTokenizer.nextToken();
				String newLine = line;
				// example ProxyServer is key, "http=markdunn:1000;socks=localhost:450"
				if(line.startsWith("\"ProxyEnable"))//$NON-NLS-1$
				{
					StringTokenizer lineTokenizer = new StringTokenizer(line,":");//$NON-NLS-1$
					newLine=lineTokenizer.nextToken();  // get ProxyEnable
					newLine=lineTokenizer.nextToken();
					if (newLine.equalsIgnoreCase("00000001"))
						comparesOK = true;

				}  //defect 82974 below - also check AutoConfigURL
				else if (line.startsWith("\"AutoConfigURL")) {
					setBadProxySettings(true);
					setProxyError(AUTO_CONFIG_URL);
					setAutoConfigURLEnabled(true);
					String autoConfigStr = "";
					autoConfigStr = findAutoConfigURLString(line);
					setAutoConfigURLString(autoConfigStr);
					break;				
				}				
			}
		return comparesOK;
	}
	
	/**
	 * @param line
	 * @return
	 */
	
	private String getProxyInfo(StringBuffer inputBuffer) 
	{
		boolean comparesOK = false;
		String proxyString = "";
		String httpString = "http=";
		String secureString = "https=";
		String socksString = "socks=";
		String equalsString = "=";
		String HTTPString = "";
		String SECUREString = "";
		int findSOCKS = -1;
		int findHTTP = -1;
		int findSECURE = -1;
		StringTokenizer fileTokenizer = new StringTokenizer(inputBuffer.toString(),"\r\n");//$NON-NLS-1$
		while(fileTokenizer.hasMoreTokens())
		{
			String line=fileTokenizer.nextToken();
			String newLine = line;
			// example ProxyServer is key, "http=markdunn:1000;https=mdunnlinux:3128;socks=localhost:450"
			// or "mdunnlinux:3128 - which means use this proxy for all protocols
			
			if(line.startsWith("\"ProxyServer"))//$NON-NLS-1$
			{
				String tmp = "\"ProxyServer\"=";
				String modifiedLine = line.substring(tmp.length());
				String [] entireString = modifiedLine.split(";");
				int numVars = entireString.length;
				for (int i =0; i < numVars; i++){
					String tmpEntireString = entireString[i].replaceAll("\"","");
					int findEqualsString = tmpEntireString.indexOf(equalsString);
					if (findEqualsString == -1) {
						proxyString = tmpEntireString;
					} 
					else {
						if (findSOCKS == -1){
							findSOCKS = tmpEntireString.indexOf(socksString);
							if (findSOCKS >= 0)
								break;
						} 
						if (findHTTP == -1) {
							findHTTP = tmpEntireString.indexOf(httpString);
							if (findHTTP >= 0)
								HTTPString = tmpEntireString;
						}
						if (findSECURE == -1) {
							findSECURE = tmpEntireString.indexOf(secureString);
							if (findSECURE >= 0)
								SECUREString = tmpEntireString;
						}
					}
				}
				if (findSOCKS >= 0) {
					setBadProxySettings(true);
					setProxyError(SOCKS_SET);
				}
				else {
					if ((findHTTP >= 0) && (findSECURE == -1)) {
						setBadProxySettings(true);
						setProxyError(HTTP_PROXY_ONLY_SET);
					}
					else if ((findHTTP == -1) && (findSECURE >= 0 )) {
						setBadProxySettings(true);
						setProxyError(SECURE_PROXY_ONLY_SET);
					}
					else if ((findHTTP >= 0) && (findSECURE >= 0)) {
						// if two string equal - it is OK
						// if different - then ERROR
						// HTTPString=http=mdunnlinux:3128 
						// SECUREString=https=mdunnlinux:3128
						// must first strip off up to '=' and then compare
						int httpEQUALS = HTTPString.indexOf("=");
						int secureEQUALS = SECUREString.indexOf("=");
						String tmpHTTP = HTTPString.substring(httpEQUALS+1);
						String tmpSECURE = SECUREString.substring(secureEQUALS+1);
						//if (HTTPString.equalsIgnoreCase(SECUREString)){
						if (tmpHTTP.equalsIgnoreCase(tmpSECURE)){
							if (tmpHTTP.startsWith("localhost")) {
								setBadProxySettings(true);
								setProxyError(HTTP_SECURE_LOCALHOST);
							}
							else {
								setBadProxySettings(false);
								setProxyError(PROXY_SETTINGS_OK);
								String tmpVal = "";
								tmpVal = HTTPString.substring(findHTTP+httpString.length());
								proxyString = tmpVal;
							}
						}
						else {
							//setBadProxySettings(true);
							//setProxyError(HTTP_SECURE_MISMATCH);
							// bugzilla 79219 mdd
							// https proxy different from http proxy 
							setBadProxySettings(false);
							secureStringGlobal = tmpSECURE;
							setSSLProxyEnabled(true);
							setProxyError(PROXY_SETTINGS_OK);
							String tmpVal = "";
							tmpVal = HTTPString.substring(findHTTP+httpString.length());
							proxyString = tmpVal;
						}
					}
				}
			}
			// defect 82974 1/18/2005
			else if (line.startsWith("\"ProxyOverride")) {
				setProxyOverrideEnabled(true);
				proxyOverrideString = findOverrideString(line);
			}
			else if (line.startsWith("\"AutoConfigURL")) {
				setAutoConfigURLEnabled(true);
				autoConfigURLString = findAutoConfigURLString(line);
			}
		}
		return proxyString;
	}

	/**
	 * @param line
	 * @return
	 */
	private String findOverrideString(String line) {
		// TODO Auto-generated method stub
		String returnStr = "";
		String tmp="\"ProxyOverride";
		String tmp2 = line.replaceAll("\"","");
		String tmp3= tmp2.replaceAll("<","");
		tmp3= tmp3.replaceAll(">","");
				
		returnStr = tmp3.substring(tmp.length());
		return returnStr;
	}
	
	private String findAutoConfigURLString(String line) {
		// TODO Auto-generated method stub
		String returnStr = "";
		String tmp="\"AutoConfigURL";
		String tmp2 = line.replaceAll("\"","");
		String tmp3= tmp2.replaceAll("<","");
		tmp3= tmp3.replaceAll(">","");
				
		returnStr = tmp3.substring(tmp.length());
		return returnStr;
	}

	/**
	 * @param string
	 */
	public void setProxyType(String string) {
		proxyType = string;
	}
	/**
	 * @return
	 */
	public String getProxyType() {
		return proxyType;
	}

	/**
	 * @param string
	 */
	public void setProxyPort(String string) {
		proxyPort = string;
	}
	/**
	 * @return
	 */
	public String getProxyPort() {
		return proxyPort;
	}

	/**
	 * @return
	 */
	public boolean isProxyEnabled() {
		return isProxyEnabled;
	}

	/**
	 * @param b
	 */
	public void setProxyEnabled(boolean b) {
		isProxyEnabled = b;
	}

	/**
	 * @return
	 */
	public String getProxyAddr() {
		return proxyAddr;
	}

	/**
	 * @param string
	 */
	public void setProxyAddr(String string) {
		proxyAddr = string;
	}

	/**
	 * @return Returns the badProxySettings.
	 */
	public boolean isBadProxySettings() {
		return badProxySettings;
	}
	/**
	 * @param badProxySettings The badProxySettings to set.
	 */
	public void setBadProxySettings(boolean badProxySettings) {
		this.badProxySettings = badProxySettings;
	}
	/**
	 * @return Returns the proxyError.
	 */
	public int getProxyError() {
		return proxyError;
	}
	/**
	 * @param proxyError The proxyError to set.
	 */
	public void setProxyError(int proxyError) {
		this.proxyError = proxyError;
	}
	/**
	 * @return Returns the sslProxyPort.
	 */
	public String getSslProxyPort() {
		return sslProxyPort;
	}
	/**
	 * @param sslProxyPort The sslProxyPort to set.
	 */
	public void setSslProxyPort(String sslProxyPort) {
		this.sslProxyPort = sslProxyPort;
	}
	/**
	 * @return Returns the sslProxy.
	 */
	public String getSslProxyAddr() {
		return sslProxyAddr;
	}
	/**
	 * @param sslProxy The sslProxy to set.
	 */
	public void setSslProxyAddr(String sslProxy) {
		this.sslProxyAddr = sslProxy;
	}
	/**
	 * @return Returns the isSSLProxyEnabled.
	 */
	public boolean isSSLProxyEnabled() {
		return isSSLProxyEnabled;
	}
	/**
	 * @param isSSLProxyEnabled The isSSLProxyEnabled to set.
	 */
	public void setSSLProxyEnabled(boolean isSSLProxyEnabled) {
		this.isSSLProxyEnabled = isSSLProxyEnabled;
	}
	/**
	 * @return Returns the isAutoConfigURLEnabled.
	 */
	public boolean isAutoConfigURLEnabled() {
		return isAutoConfigURLEnabled;
	}
	/**
	 * @param isAutoConfigURLEnabled The isAutoConfigURLEnabled to set.
	 */
	public void setAutoConfigURLEnabled(boolean isAutoConfigURLEnabled) {
		this.isAutoConfigURLEnabled = isAutoConfigURLEnabled;
	}
	
	/**
	 * @return Returns the isProxyOverrideEnabled.
	 */
	public boolean isProxyOverrideEnabled() {
		return isProxyOverrideEnabled;
	}
	/**
	 * @param isProxyOverrideEnabled The isProxyOverrideEnabled to set.
	 */
	public void setProxyOverrideEnabled(boolean isProxyOverrideEnabled) {
		this.isProxyOverrideEnabled = isProxyOverrideEnabled;
	}
	/**
	 * @return Returns the autoConfigURLString.
	 */
	public String getAutoConfigURLString() {
		return autoConfigURLString;
	}
	/**
	 * @param autoConfigURLString The autoConfigURLString to set.
	 */
	public void setAutoConfigURLString(String autoConfigURLString) {
		this.autoConfigURLString = autoConfigURLString;
	}
	/**
	 * @return Returns the proxyOverrideString.
	 */
	public String getProxyOverrideString() {
		return proxyOverrideString;
	}
	/**
	 * @param proxyOverrideString The proxyOverrideString to set.
	 */
	public void setProxyOverrideString(String proxyOverrideString) {
		this.proxyOverrideString = proxyOverrideString;
	}
}
