/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: NewActionGroup.java,v 1.12 2005/02/16 22:20:28 qiyanli Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
/*
 * Created on Nov 6, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package org.eclipse.hyades.log.ui.internal.navigator;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;

import org.eclipse.hyades.log.ui.internal.LogUIPlugin;
import org.eclipse.hyades.log.ui.internal.wizards.NewCorrelationWizard;
import org.eclipse.hyades.models.hierarchy.TRCAgentProxy;
import org.eclipse.hyades.ui.HyadesUIPlugin;
import org.eclipse.hyades.ui.extension.INavigatorContribution;
import org.eclipse.hyades.ui.internal.logicalfolder.LogicalFolder;
import org.eclipse.hyades.ui.internal.navigator.Navigator;
import org.eclipse.hyades.ui.internal.navigator.action.NavigatorActionGroup;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.actions.NewWizardMenu;
import org.eclipse.ui.views.navigator.ResourceNavigatorMessages;

/**
 * @author apnan
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class NewActionGroup extends NavigatorActionGroup {

	private static final int WIZARD_WIDTH = 500;
	private static final int WIZARD_HEIGHT = 400;

	private Action newSymptomDBAction;
	private Action newCorrelationAction;

	/**
	 * Constructor for CreateGroupAction
	 * @param testNavigator
	 */
	public NewActionGroup(Navigator navigator)
	{
		super(navigator);
	}
	/* (non-Javadoc)
	 * @see org.eclipse.hyades.ui.internal.navigator.action.NavigatorActionGroup#createActions()
	 */
	protected void createActions() {

		if (LogUIPlugin.getDefault().getNewSymDBWizard()!=null) {
			newSymptomDBAction = new Action() {
				public void run() {
					INewWizard wiz = LogUIPlugin.getDefault().getNewSymDBWizard();
					wiz.init(HyadesUIPlugin.getInstance().getWorkbench(), getNavigator().getStructuredSelection());
					WizardDialog dialog = new WizardDialog(getNavigator().getSite().getWorkbenchWindow().getShell(), wiz);
					dialog.setPageSize(WIZARD_WIDTH, WIZARD_HEIGHT);
					dialog.open();
				}
			};
			newSymptomDBAction.setText(LogUIPlugin.getResourceString("STR_NEW_SYMPTOM_DATABASE_TOOLTIP"));	
			newSymptomDBAction.setToolTipText(LogUIPlugin.getResourceString("STR_NEW_SYMPTOM_DATABASE_TOOLTIP"));
			newSymptomDBAction.setImageDescriptor(getImageDescriptor("obj16/new_symptomdb_obj.gif"));
			newSymptomDBAction.setHoverImageDescriptor(getImageDescriptor("obj16/new_symptomdb_obj.gif"));
		}
		
		newCorrelationAction = new Action() {
			public void run() {
				NewCorrelationWizard wiz = new NewCorrelationWizard();
				wiz.init(HyadesUIPlugin.getInstance().getWorkbench(), getNavigator().getStructuredSelection());
				WizardDialog dialog = new WizardDialog(getNavigator().getSite().getWorkbenchWindow().getShell(), wiz);
				dialog.setPageSize(WIZARD_WIDTH, WIZARD_HEIGHT);
				dialog.open();
			}
		};
		newCorrelationAction.setText(LogUIPlugin.getResourceString("STR_NEW_CORR_MENUITEM"));
		newCorrelationAction.setToolTipText(LogUIPlugin.getResourceString("STR_NEW_CORR_TOOLTIP"));
		newCorrelationAction.setImageDescriptor(getImageDescriptor("etool16/newcorrelation_wiz.gif"));
		newCorrelationAction.setHoverImageDescriptor(getImageDescriptor("etool16/newcorrelation_wiz.gif"));	
	}
	

	/**
	 * Returns the image descriptor from the eclipse UI plugin with the given relative 
	 * path.
	 * @return ImageDescriptor;
	 */
	protected ImageDescriptor getImageDescriptor(String relativePath)
	{
		try
		{
			URL installURL = new URL(LogUIPlugin.getDefault().getBundle().getEntry("/"), "icons/full/");
			URL url = new URL(installURL, relativePath);
			return ImageDescriptor.createFromURL(url);
		}
		catch(MalformedURLException e)
		{
			return ImageDescriptor.getMissingImageDescriptor();
		}
	}
	
	/**
	 * @see org.eclipse.ui.actions.ActionGroup#fillContextMenu(org.eclipse.jface.action.IMenuManager)
	 */
	public void fillContextMenu(IMenuManager menu)
	{
		IStructuredSelection structuredSelection = (IStructuredSelection)getContext().getSelection();

        
		if(getNavigator().isShowingFolders())
		{
			if(!hasLogicalFolder(structuredSelection))
			{
				MenuManager newMenu = new MenuManager(ResourceNavigatorMessages.getString("ResourceNavigator.new"));
				menu.appendToGroup(INavigatorContribution.GROUP_ADDITIONS, newMenu);

				new NewWizardMenu(newMenu, getNavigator().getSite().getWorkbenchWindow(), false);
				
				if (newSymptomDBAction!=null) {
					newMenu.insert(0, new ActionContributionItem(newSymptomDBAction));
					newMenu.insert(1, new Separator());
				}
			}
		}
		else
		{
			MenuManager newMenu = null;
			if(structuredSelection.size()>0){
				boolean selAgents = true;
				for (Iterator iter = structuredSelection.iterator(); iter.hasNext();) {
					if(!(iter.next() instanceof TRCAgentProxy)){
						selAgents = false;
						break;
					}					
				}
				if(selAgents){
					newMenu = new MenuManager(ResourceNavigatorMessages.getString("ResourceNavigator.new"));
					menu.appendToGroup(INavigatorContribution.GROUP_ADDITIONS, newMenu);							
					newMenu.add(newCorrelationAction);					
				}
			}

			if(structuredSelection.size() == 1)
			{
				Object selection = structuredSelection.getFirstElement();
				if(selection instanceof LogicalFolder)
				{
					LogicalFolder logicalFolder = (LogicalFolder)selection;
					if(logicalFolder.getParent() == null)
					{						

						String name = logicalFolder.getName();
						if(name.equals(LogUIPlugin.getResourceString("LOG_NAV_SDFLD"))){
							newMenu = new MenuManager(ResourceNavigatorMessages.getString("ResourceNavigator.new"));
							menu.appendToGroup(INavigatorContribution.GROUP_ADDITIONS, newMenu);
							if (newSymptomDBAction!=null) 
								newMenu.add(newSymptomDBAction);
						}
							
						if(name.equals(LogUIPlugin.getResourceString("LOG_NAV_CORRFLD"))){	
							newMenu = new MenuManager(ResourceNavigatorMessages.getString("ResourceNavigator.new"));
							menu.appendToGroup(INavigatorContribution.GROUP_ADDITIONS, newMenu);							
							newMenu.add(newCorrelationAction);
						}
					}
				}
			}
		}
	}
	
		/**
		 * Returns <code>true</code> is there is one logical folder in the 
		 * given selection.
		 * @param structuredSelection
		 * @return boolean
		 */
		protected boolean hasLogicalFolder(IStructuredSelection structuredSelection)
		{
			for (Iterator i = structuredSelection.iterator(); i.hasNext();)
			{
				if(i.next() instanceof LogicalFolder)
					return true;
			}
		
			return false;
		}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.actions.ActionGroup#fillActionBars(org.eclipse.ui.IActionBars)
	 */
	public void fillActionBars(IActionBars actionBars) {		
		
		IToolBarManager toolbar = actionBars.getToolBarManager();		
		toolbar.add(new Separator());
		toolbar.add(newCorrelationAction);	
		if (newSymptomDBAction!=null) 
			toolbar.add(newSymptomDBAction);
		
	}

}
