/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: OpenLogViewAction.java,v 1.15 2005/02/16 22:21:32 qiyanli Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/

package org.eclipse.hyades.sd.logc.internal.actions;

import org.eclipse.core.resources.IResourceStatus;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.hyades.models.hierarchy.CorrelationContainerProxy;
import org.eclipse.hyades.models.hierarchy.util.PerfUtil;
import org.eclipse.hyades.sd.logc.SDLogcConstants;
import org.eclipse.hyades.sd.logc.SDLogcPlugin;
import org.eclipse.hyades.sd.logc.internal.loader.LogInteractions;
import org.eclipse.hyades.trace.ui.HyadesUtil;
import org.eclipse.hyades.trace.ui.UIPlugin;
import org.eclipse.hyades.uml2sd.ui.load.LoadersManager;
import org.eclipse.hyades.uml2sd.ui.view.SDView;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.*;


public class OpenLogViewAction
	extends Action
	implements IWorkbenchWindowActionDelegate, IViewActionDelegate
{

	public OpenLogViewAction() {
		super("");

	}

	public OpenLogViewAction(String label) {
		super(label);
	}

	public OpenLogViewAction(String label, ImageDescriptor image) {
		super(label, image);
	}

	public void run() {
		//System.out.println("OpenSDViewAction.run");
		IWorkbenchWindow window =
		SDLogcPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow();

		try {
			IWorkbenchPage persp = UIPlugin.getActivePage();
			PerfUtil p = new PerfUtil("openLogInteractions",true);
			SDView view = view = (SDView) persp.showView(SDLogcConstants.LogInteractionsViewID); //$NON-NLS-1$;
			String menuItemName=getText();
			if(menuItemName.equals(SDLogcPlugin.getString("STR_MENU_LOR"))){
				LoadersManager.getLoadersManager().createLoader("org.eclipse.hyades.sd.logc.internal.loader.LogInteractions", getClass().getClassLoader(), view);
				EObject sel = HyadesUtil.getMofObject();
				if(sel instanceof CorrelationContainerProxy){	
					((LogInteractions)LoadersManager.getLoadersManager().getCurrentLoader(SDLogcConstants.LogInteractionsViewID)).generateLogInteractions(sel,false);
				}else{
					((LogInteractions)LoadersManager.getLoadersManager().getCurrentLoader(SDLogcConstants.LogInteractionsViewID)).generateLogInteractions(sel,true);				
				}
			}else if(menuItemName.equals(SDLogcPlugin.getString("STR_MENU_LOT"))){
				LoadersManager.getLoadersManager().createLoader("org.eclipse.hyades.sd.logc.internal.loader.LogThreadInteractions", getClass().getClassLoader(), view);
				EObject sel = HyadesUtil.getMofObject();
				if(sel instanceof CorrelationContainerProxy){	
					((LogInteractions)LoadersManager.getLoadersManager().getCurrentLoader(SDLogcConstants.LogInteractionsViewID)).generateLogInteractions(sel,false);
				}else{
					((LogInteractions)LoadersManager.getLoadersManager().getCurrentLoader(SDLogcConstants.LogInteractionsViewID)).generateLogInteractions(sel,true);				
				}				
			}
			p.stopAndPrintStatus();
		} catch (Exception e) {
			String msg = SDLogcPlugin.getString("STR_LOAD_SD_ERR_");

			Status err =
				new Status(
					Status.WARNING,
					ResourcesPlugin.PI_RESOURCES,
					IResourceStatus.INTERNAL_ERROR,
					e.toString(),
					null);

			ErrorDialog.openError(window.getShell(), msg, "",err);

			e.printStackTrace();
		}

	}

	public void dispose() {
	}
	
	public void init(IWorkbenchWindow window) {
	}

	  
	//---- IWorkbenchWindowActionDelegate ------------------------------------------------

	public void run(IAction action) {
		run();
	}

	public void selectionChanged(IAction action, ISelection selection)
    {
	}
	
	public void init(IViewPart view)
	{
	}
	
}

