/**********************************************************************
 * Copyright (c) 2005 Scapa Technologies Limited and others
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * Scapa Technologies Limited - Initial API and implementation
 **********************************************************************/

package org.eclipse.hyades.statistical.ui.monitoractions.internal.logfile;

import java.io.File;

import org.eclipse.hyades.statistical.ui.EditorPlugin;
import org.eclipse.hyades.statistical.ui.editor.internal.InvalidDataException;
import org.eclipse.hyades.statistical.ui.widgets.alert.internal.AlertAction;
import org.eclipse.hyades.statistical.ui.widgets.alert.internal.AlertActionControl;
import org.eclipse.hyades.statistical.ui.widgets.internal.GData;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Text;

public class LogFileControl extends AlertActionControl implements SelectionListener {
LogFileAlertAction action;

Label label;
Text path;
Button browse;

	public LogFileControl(Composite c, LogFileAlertAction action) {
		super(c,0);
		this.action = action;
		
		initGraphics();
		initEventHandling();
		
		readFromAction(action);
	}
	
	private void initGraphics() {
		setLayout(new GridLayout(2,false));
		
		label = new Label(this,0);
		label.setText(EditorPlugin.getString("ALERT_ACTION_LOGFILE_PATH"));
		label.setLayoutData(GData.getGridData(true,false,-1,-1,2,1,GridData.FILL,GridData.FILL));
		
		path = new Text(this,SWT.BORDER);
		path.setText("              ");
		path.setLayoutData(GData.getGridData(true,false,-1,-1,1,1,GridData.FILL,GridData.FILL));
		
		browse = new Button(this,0);
		browse.setText(EditorPlugin.getString("BROWSE"));
		browse.setLayoutData(GData.getGridData(false,false,-1,-1,1,1,GridData.FILL,GridData.END));
	}
	
	private void initEventHandling() {
		browse.addSelectionListener(this);
	}
	
	public void readFromAction(AlertAction tmp) {
		LogFileAlertAction lfaction = (LogFileAlertAction)tmp;
		path.setText(lfaction.path);
	}
	
	public void writeToAction(AlertAction tmp) throws InvalidDataException {
		if (path.getText().length() == 0) {
			throw new InvalidDataException(EditorPlugin.getString("ERROR_EMPTY_ALERT_LOGFILE_PATH"));
		}
		LogFileAlertAction lfaction = (LogFileAlertAction)tmp;
		lfaction.path = path.getText();
	}
	public AlertAction writeToClonedAction() throws InvalidDataException {
		LogFileAlertAction clone = (LogFileAlertAction)action.clone();
		writeToAction(clone);
		return clone;
	}
	public AlertAction writeToOriginalAction() throws InvalidDataException {
		writeToAction(action);
		return action;
	}

	public String getDescription() {
		return EditorPlugin.getString("ALERT_ACTION_NAME_LOGFILE");
	}
	
	////////////////////////
	// selection listener
	///////////////////////
	public void widgetDefaultSelected(SelectionEvent e) {
		widgetSelected(e);
	}
	public void widgetSelected(SelectionEvent e) {
		widgetSelected(e.getSource());
	}
	public void widgetSelected(Object o) {
		if (o == browse) {
			FileDialog fd = new FileDialog(getShell());
			fd.setFileName(path.getText());
			String tmp = fd.open();
			
			if (tmp != null) {
				if (tmp.length() != 0) {
					if (new File(tmp).exists()) {
						MessageBox mbox = new MessageBox(getShell(),SWT.YES|SWT.NO|SWT.CANCEL);
						mbox.setText(EditorPlugin.getString("ALERT_ACTION_LOGFILE_PATHEXISTS"));
						mbox.setMessage(EditorPlugin.getString("ALERT_ACTION_LOGFILE_PATHCONFIRM"));
						if (mbox.open() == SWT.YES) {
							path.setText(tmp);
						}
					} else {
						path.setText(tmp);
					}
				}
			}
			
		}
	}
	
}

