/*******************************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: ManualTestLaunchConfigurationDelegate.java,v 1.3 2005/02/16 22:21:49 qiyanli Exp $
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.hyades.test.manual.internal.launch;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.hyades.execution.core.IExecutor;
import org.eclipse.hyades.execution.harness.TestExecutionHarness;
import org.eclipse.hyades.models.common.facades.behavioral.ITest;
import org.eclipse.hyades.models.common.facades.behavioral.ITestSuite;
import org.eclipse.hyades.models.common.testprofile.TPFDeployment;
import org.eclipse.hyades.models.common.testprofile.TPFTest;
import org.eclipse.hyades.models.common.testprofile.TPFTestCase;
import org.eclipse.hyades.models.common.testprofile.TPFTestSuite;
import org.eclipse.hyades.models.common.util.SaveManager;
import org.eclipse.hyades.test.manual.internal.codegen.ManualGenerator;
import org.eclipse.hyades.test.ui.TestUI;
import org.eclipse.hyades.test.ui.TestUIPlugin;
import org.eclipse.hyades.test.ui.launch.delegates.BasicTestLaunchConfigurationDelegate;
import org.eclipse.hyades.ui.internal.util.StringUtil;

public class ManualTestLaunchConfigurationDelegate extends BasicTestLaunchConfigurationDelegate {
	
	public final static String LAUNCH_CONFIGURATION_TYPE =  "org.eclipse.hyades.test.manual.launch.manualTest"; //$NON-NLS-1$

	/* (non-Javadoc)
	 * @see org.eclipse.hyades.test.ui.launch.delegates.AbstractLaunchConfigurationDelegate#invokeTestExecutionHarness(org.eclipse.debug.core.ILaunchConfiguration, java.lang.String, java.lang.StringBuffer)
	 */
	public IExecutor invokeTestExecutionHarness(
			ILaunchConfiguration configuration, String mode,
			StringBuffer errorMessages) throws CoreException 
	{
		// Compute parameters
    	TPFTest test = getLaunchedTest(configuration);
    	TPFTestSuite testSuite = null;
    	TPFTest subTest = null;
    	if (test instanceof TPFTestCase) {
    		subTest = test;
    		testSuite = ((TPFTestCase)subTest).getTestSuite();
    	} else if (test instanceof TPFTestSuite) {
    		testSuite = (TPFTestSuite)test;
    		subTest = test;
    	}
    	
		Resource resource = ((TPFTestSuite)testSuite).eResource();
		boolean wasModified = resource.isModified();
		
		testSuite.getImplementor().setResource(createTestScript(testSuite));
		
		if(!wasModified)
			resource.setModified(false);
    	
    	TPFDeployment deployment = getDeployment(configuration);	    	
    	String tehName = getTestExecutionHistoryName(configuration);
    	String tehLocation = getTestExecutionHistoryLocation(configuration).getFullPath().toString();
    	String port = TestUIPlugin.getInstance().getPreferenceStore().getString(TestUI.LOCALHOST_PORT);
    	
		TestExecutionHarness teh = new TestExecutionHarness();
		IExecutor executor = teh.launchTest(
			(ITestSuite)testSuite,
			(ITest)subTest,
			deployment,
			port,
			tehLocation,
			tehName,
			true,
			false,
			errorMessages);
		SaveManager.getInstance().start(TestUIPlugin.getInstance().getPreferenceStore().getLong(TestUI.SAVE_INTERVAL));
		return executor;

	}
	
	protected String createTestScript(ITestSuite testSuite)
	{
		String testScript = ManualGenerator.generateScript(testSuite);
		if(testScript == null)
			return null;
			
		testScript = StringUtil.replace(testScript, "\r", "");
		return StringUtil.replace(testScript, "\n", "");
	}
	
}
