/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: PDCoreUtil.java,v 1.41 2005/04/29 19:43:31 ewchan Exp $
 *
 * Contributors:
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.hyades.trace.ui.internal.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.UnknownHostException;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.Vector;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceStatus;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.debug.core.DebugEvent;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.model.IProcess;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.hyades.internal.execution.local.common.CustomCommand;
import org.eclipse.hyades.internal.execution.local.common.Options;
import org.eclipse.hyades.internal.execution.local.control.Agent;
import org.eclipse.hyades.internal.execution.local.control.AgentConfiguration;
import org.eclipse.hyades.internal.execution.local.control.AgentConfigurationEntry;
import org.eclipse.hyades.internal.execution.local.control.AgentFactory;
import org.eclipse.hyades.internal.execution.local.control.InactiveAgentException;
import org.eclipse.hyades.internal.execution.local.control.InactiveProcessException;
import org.eclipse.hyades.internal.execution.local.control.Node;
import org.eclipse.hyades.internal.execution.local.control.NodeFactory;
import org.eclipse.hyades.internal.execution.local.control.NotConnectedException;
import org.eclipse.hyades.internal.execution.local.control.Process;
import org.eclipse.hyades.internal.execution.local.control.ProcessFactory;
import org.eclipse.hyades.internal.execution.local.control.ProcessImpl;
import org.eclipse.hyades.internal.execution.local.control.ProcessListener;
import org.eclipse.hyades.internal.execution.local.control.Variable;
import org.eclipse.hyades.loaders.util.LoadersUtils;
import org.eclipse.hyades.models.hierarchy.HierarchyFactory;
import org.eclipse.hyades.models.hierarchy.TRCAgent;
import org.eclipse.hyades.models.hierarchy.TRCAgentProxy;
import org.eclipse.hyades.models.hierarchy.TRCCollectionMode;
import org.eclipse.hyades.models.hierarchy.TRCConfiguration;
import org.eclipse.hyades.models.hierarchy.TRCFilter;
import org.eclipse.hyades.models.hierarchy.TRCMonitor;
import org.eclipse.hyades.models.hierarchy.TRCNode;
import org.eclipse.hyades.models.hierarchy.TRCOption;
import org.eclipse.hyades.models.hierarchy.TRCProcessProxy;
import org.eclipse.hyades.models.hierarchy.util.HierarchyResourceSetImpl;
import org.eclipse.hyades.security.util.ConnectUtil;
import org.eclipse.hyades.trace.internal.ui.PDContentProvider;
import org.eclipse.hyades.trace.internal.ui.PDPerspective;
import org.eclipse.hyades.trace.internal.ui.PDProjectExplorer;
import org.eclipse.hyades.trace.internal.ui.TraceConstants;
import org.eclipse.hyades.trace.internal.ui.TraceUIManager;
import org.eclipse.hyades.trace.ui.HyadesConstants;
import org.eclipse.hyades.trace.ui.ProfileEvent;
import org.eclipse.hyades.trace.ui.UIPlugin;
import org.eclipse.hyades.trace.ui.internal.launcher.IProfileLaunchConfigurationConstants;
import org.eclipse.hyades.trace.ui.internal.launcher.TraceArguments;
import org.eclipse.hyades.trace.ui.internal.piclient.PIAttachListener;
import org.eclipse.hyades.trace.ui.internal.piclient.PIProcessListener;
import org.eclipse.hyades.trace.ui.internal.piclient.XMLTraceDataProcessor;
import org.eclipse.hyades.ui.HyadesUI;
import org.eclipse.hyades.ui.HyadesUIPlugin;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;



public class PDCoreUtil {
    //~ Methods ------------------------------------------------------------------------------------

    /**
     * attach to an agent
     *
     * @deprecated user should call attach(Sting,Vector,String,String)
     *  with portNumber specific instead.
     **/
    public static final Vector attach(String hostName, Vector agents, Vector options, ArrayList filters, String selectedProject, String selectedMonitor) {
        return attach(hostName, agents, options, filters, selectedProject, selectedMonitor, "10002", null, null);
    }

    public static final Vector attach(String hostName, Vector agents, Vector options, ArrayList filters, String selectedProject, String selectedMonitor, String portNumber, String profileFile, ILaunch launch) {
        IPreferenceStore store = UIPlugin.getDefault().getPreferenceStore();
    	switchPerspective();

        try {
            Node fnode = NodeFactory.createNode(hostName);

            hostName = fnode.getInetAddress().getHostName();
        } catch (UnknownHostException exc) {
        }

        //create the monitor
        IContainer container = createContainer(new Path(selectedProject));
        TRCMonitor monitor = createMonitor(container, selectedMonitor);

        //create the node   
        TRCNode node = createNode(monitor, hostName, portNumber);

        TRCProcessProxy process = null;
        Vector trcAgentList = new Vector();

        for (int idx = 0; idx < agents.size(); idx++) {
            Agent agent = (Agent) agents.elementAt(idx);
            Process pInfo = agent.getProcess();

            final IProcess adapter = new ProcessAdapter(pInfo, launch, true, 0);
    		TraceUIManager.getTraceUIManager().registerLaunchProcess(adapter);
            pInfo.addProcessListener(new ProcessListener() {
            	public void processLaunched(Process p) {
            	}
            	public void processExited(Process p) {
            		TraceUIManager.getTraceUIManager().deregisterLaunchProcess(adapter);
            		DebugPlugin.getDefault().fireDebugEventSet(new DebugEvent[] {
               			new DebugEvent(adapter, DebugEvent.TERMINATE)
            		});
            	}
            });

            AgentConfiguration ac = agent.getConfiguration();
            if (ac == null) {
                agent.setConfiguration(new AgentConfiguration());
            }

            agent.getConfiguration().clear();
            if ((idx == 0) && (profileFile != null) && !profileFile.equals("")) {
                agent.setProfileFile(profileFile);
            } else
            	agent.setProfileFile("");
            addDefaultProfileData(agent, filters);
            addDefaultOptionsData(agent, options);

            process = createProcess(node, pInfo);
            process.eAdapters().add(new ProcessProxyAdapter());

            // add the mapping between TRCProcessProxy -> IProcess
            ProcessMap.put(process, adapter);
            
            try {
                TRCAgentProxy agentProxy = createUniqueAgent(process, agent);

                activate(agentProxy);
                doAttach(agent, agentProxy);
               
                trcAgentList.add(agentProxy);

                final ProfileEvent event = UIPlugin.getDefault().getProfileEvent();
                event.setSource(agentProxy);
                event.setType(ProfileEvent.ATTACH);

                Display.getDefault().syncExec(new Runnable() {
					public void run() {
		                UIPlugin.getDefault().notifyProfileEventListener(event);
					}
                });
            } catch (Exception exc) {
                exc.printStackTrace();
            }
        }

        return trcAgentList;
    }

	public static void activate(TRCAgentProxy agentProxy) {
		//deactivate previous agents
		for (Iterator iter = agentProxy.getProcessProxy().getAgentProxies().iterator(); iter.hasNext();) {
			TRCAgentProxy element = (TRCAgentProxy) iter.next();
			try {
				if(agentProxy.getName().equals(element.getName()) && agentProxy.getType().equals(element.getType()))
				{
					element.setActive(false);
					element.setAttached(false);
				}
			} catch (NullPointerException e) {
				// ignore null pointer exceptions
			}
		}
		agentProxy.setActive(true);
		agentProxy.setAttached(true);
		
	}

	public static final void detachAgent(TRCAgentProxy agent) throws InactiveAgentException, InactiveProcessException {
        TRCProcessProxy process = agent.getProcessProxy();
        String hostName = process.getNode().getName();
        Node node = PDCoreUtil.profileConnect(hostName, String.valueOf(process.getNode().getPort()));

        if (node != null) {
            Object instance = LoadersUtils.locateAgentInstance(agent);

            if ((instance != null) && instance instanceof Agent) {
                Agent a = (Agent) instance;
//				AgentListener al = (AgentListener)LoadersUtils.locateAgentListener(agent);
//				a.removeAgentListener(al);

                if (a.isAttached()) {
                    a.detach();
                }
            }
        }

        agent.setMonitored(false);
        agent.setAttached(false);

        LoadersUtils.deregisterDataProcessor(agent);
//		LoadersUtils.deregisterAgentInstance(agent);
//		LoadersUtils.deregisterAgentListener(agent);

        if (agent.isToProfileFile()) {
            XMLTraceDataProcessor processor = (XMLTraceDataProcessor) LoadersUtils.locateDataProcessor(agent);

            if (processor != null) {
                processor.endTrace(processor.getWriter());
                processor.setProfileFileName(null);

                //					  processor.setProfilingFile(null);
                processor.setWriter(null);
            }
        }
    }

    public static TRCAgentProxy createAgent(TRCProcessProxy process, Agent a) {
        TRCAgentProxy agentProxy = createAgent(process, a, "");

        //	setSessionStartTime(a.get, agentProxy);
        return agentProxy;
    }

	public static TRCAgentProxy getCorrespondingAgent(TRCProcessProxy processProxy,Agent ag,boolean activeProxy) {
		Iterator i = processProxy.getAgentProxies().iterator();

		while (i.hasNext()) {
			TRCAgentProxy agentProxy = (TRCAgentProxy) i.next();

			if (agentProxy.eIsProxy()) {
				continue;
			}

			if ((agentProxy != null) && agentProxy.getName().equals(ag.getName()) && (agentProxy.getRuntimeId() == null || agentProxy.getRuntimeId() == ag.getUUID()) && agentProxy.isActive()==activeProxy) {
				return agentProxy;
			}
		}

		return null;
	}

    public static TRCAgentProxy createAgent(TRCProcessProxy processProxy, Agent agent, String sessionId) {
        TRCAgentProxy agentProxy = getCorrespondingAgent(processProxy,agent,false);
        if(agentProxy!=null)
        	return agentProxy;
//        String name = a.getName();

//        EList agents = process.getAgentProxies();
//        Iterator i = agents.iterator();
//
//        while (i.hasNext()) {
//            agent = (TRCAgentProxy) i.next();
//
//            if (agent.eIsProxy()) {
//                continue;
//            }
//
//            if (agent.getName().equals(name)) {
//                return agent;
//            }
//        }

        return createAgentNoCheck(processProxy, agent, sessionId);
    }

    /**
     * Insert the method's description here.
     * Creation date: (10/05/2000 10:03:14 AM)
     * @param path com.ibm.itp.common.IPath
     */
    public static final IContainer createContainer(IPath path) {
        IContainer container = null;
        IWorkspace workbench = UIPlugin.getPluginWorkbench();
        int segCount = path.segmentCount();

        for (int idx = 0; idx < segCount; idx++) {
            String seg = path.segment(idx);

            if (idx == 0) { //project

                IProject project = workbench.getRoot().getProject(path.uptoSegment(idx + 1).toString());

                if ((project == null) || !project.exists()) { //create the project

                    try {
                        project.create(null);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }

                try {
                    project.open(null);
                } catch (Exception e) {
                }

                container = project;
            } else // (idx > 1)
             { //folder

                IFolder folder = workbench.getRoot().getFolder(path.uptoSegment(idx + 1));

                if ((folder == null) || !folder.exists()) { //create the folder

                    try {
                        folder.create(false, true, null);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }

                container = folder;
            }
        }

        try {
            container.getProject().refreshLocal(IResource.DEPTH_INFINITE, null);
        } catch (CoreException exc) {
            exc.printStackTrace();
        }

        return container;
    }

    public static final TRCMonitor createMonitor(IContainer res, String name) {
        IWorkspace workbench = res.getWorkspace();

        TRCMonitor monitor = null;
        ArrayList monitors = PDContentProvider.getMonitors(res);

        for (int idx = 0; idx < monitors.size(); idx++) {
            Object mon = monitors.get(idx);

            if (mon instanceof TRCMonitor && ((TRCMonitor) mon).getName().equals(name)) {
                return (TRCMonitor) mon;
            }
        }

        String monitorName = TString.change(name, " ", "");
        String monitorFileName = monitorName + "." + TraceConstants.MONITOR_EXT;

        IPath filePath = res.getFullPath().append(monitorFileName);

        URI uri = URI.createURI("platform:/resource" + filePath.toString());
        Resource monDoc = Resource.Factory.Registry.INSTANCE.getFactory(uri).createResource(uri);
        monDoc.setModified(true);
        EList monExt = monDoc.getContents();

        UIPlugin.getDefault().getResourceSet().getResources().add(monDoc); // prevents reloading later

        HierarchyFactory factory = UIPlugin.getDefault().getPerftraceFactory();

        monitor = factory.createTRCMonitor();
        monitor.setName(name);

//        //*** adding support for multiple files
//        SaveUtil.addDocument(monDoc);

        monExt.add(monitor);

        PDContentProvider.addMonitor(res, monitor);

        return monitor;
    }

    public static final TRCNode createNode(TRCMonitor monitor, String name, String portNumber) {
        String ipAddress = "";

        try {
            Node fnode = NodeFactory.createNode(name);

            name = fnode.getInetAddress().getHostName();
            ipAddress = fnode.getInetAddress().getHostAddress();
        } catch (UnknownHostException exc) {
        }

        Object[] nodes = monitor.getNodes().toArray();

        for (int idx = 0; idx < nodes.length; idx++) {
            TRCNode node = (TRCNode) nodes[idx];

            if (node.eIsProxy()) {
                continue;
            }

            if (node.getName().equals(name)) {
                ((TRCNode) node).setPort(Integer.parseInt(portNumber));

                return (TRCNode) node;
            }
        }

        TRCNode node = null;

        String nodeName = TString.change(monitor.getName(), " ", "") + "_" + TString.change(name, " ", "");
        String nodeFileName = nodeName + "." + TraceConstants.NODE_EXT;

        String monPath = monitor.eResource().getURI().toString();
        IPath path = new Path(monPath);

        if (path.segmentCount() > 1) {
            monPath = path.removeLastSegments(1).toString();
        }

        IPath filePath = new Path(monPath).append(nodeFileName);

        URI uri = URI.createURI(filePath.toString());
        Resource nodeDoc = Resource.Factory.Registry.INSTANCE.getFactory(uri).createResource(uri);
        nodeDoc.setModified(true);
        EList nodeExt = nodeDoc.getContents();

        UIPlugin.getDefault().getResourceSet().getResources().add(nodeDoc); // prevents reloading later

        HierarchyFactory factory = UIPlugin.getDefault().getPerftraceFactory();

        node = factory.createTRCNode();
        node.setName(name);
        node.setIpAddress(ipAddress);
        node.setMonitor(monitor);
        node.setPort(Integer.parseInt(portNumber));
        nodeExt.add(node);

//        //*** adding support for multiple files
//        SaveUtil.addDocument(nodeDoc);

        EList mon_nodes = monitor.getNodes();

        try {
            mon_nodes.add(node);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return node;
    }

    public static TRCProcessProxy createProcess(TRCNode node, Process pInfo) {
        try {
            TRCProcessProxy process = null;
            EList processes = node.getProcessProxies();
            Iterator i = processes.iterator();

            while (i.hasNext()) {
                process = (TRCProcessProxy) i.next();

                if (process.eIsProxy()) {
                    continue;
                }

                if (String.valueOf(process.getPid()).equals(pInfo.getProcessId())) {
                    if ((process.getRuntimeId() != null) && process.getRuntimeId().equals(pInfo.getUUID())) {
                        return process;
                    }

                    //check if the process is active; if not, this is
                    //a different process whith same id
                    Iterator agents = process.getAgentProxies().iterator();

                    while (agents.hasNext()) {
                        TRCAgentProxy agent = (TRCAgentProxy) agents.next();

                        if (agent.eIsProxy()) {
                            continue;
                        }

                        if (agent.isActive()) {
                            return process;
                        }
                    }
                }
            }

            TRCMonitor monitor = node.getMonitor();

            String rID = (new Date()).toString();

            rID = TString.change(rID, " ", "");
            rID = TString.change(rID, ":", "");

            String processName = monitor.getName() + "_" + node.getName() + "_" + pInfo.getProcessId() + "_" + rID;

            String fileName = processName + "." + TraceConstants.PROCESS_EXT;

            String monPath = node.eResource().getURI().toString();
            IPath path = new Path(monPath);

            if (path.segmentCount() > 1) {
                monPath = path.removeLastSegments(1).toString();
            }

            IPath filePath = new Path(monPath).append(fileName);

            URI uri = URI.createURI(filePath.toString());

            Resource pDoc = Resource.Factory.Registry.INSTANCE.getFactory(uri).createResource(uri);
            pDoc.setModified(true);
            EList pExt = pDoc.getContents();

            UIPlugin.getDefault().getResourceSet().getResources().add(pDoc); // prevents reloading later

//            // adding support for multiple files
//            SaveUtil.addDocument(pDoc);

            HierarchyFactory factory = UIPlugin.getDefault().getPerftraceFactory();

            //create process element
            process = factory.createTRCProcessProxy();
            process.setPid(Integer.parseInt(pInfo.getProcessId()));

            /* Ensure the runtime UUID is set on the process object */
            process.setRuntimeId(pInfo.getUUID());

            if (pInfo.getName() != null) {
                process.setName(pInfo.getName());
            } else {
                process.setName(pInfo.getExecutable());
            }

            process.setLocation(((ProcessImpl) pInfo).getlocation());
            process.setActive(true);
            process.setLaunchMode(0); //attach                       

            process.setNode(node);
            pExt.add(process);

            return process;
        } catch (InactiveProcessException exc) {
            exc.printStackTrace();
        }

        return null;
    }

    public static void detachAgents() {
    	
        for (Iterator e = HierarchyResourceSetImpl.getInstance().getResources().iterator(); e.hasNext();) {
            Resource res = ((Resource) e.next());
            EList ext = res.getContents();
            Iterator i = ext.iterator();

            while (i.hasNext()) {
                Object obj = i.next();

				TRCAgentProxy agent = null;
				if(obj instanceof TRCAgent)
					agent = ((TRCAgent)obj).getAgentProxy();
				else if(obj instanceof TRCAgentProxy)
                   agent = (TRCAgentProxy) obj;					
					
                if (agent != null) {
                    TRCProcessProxy process = agent.getProcessProxy();

                    if ((process == null) || !process.isActive() || !agent.isActive()) {
                        continue;
                    }
					
                    try {
                        Object instance = LoadersUtils.locateAgentInstance(agent);

                        if ((instance != null) && instance instanceof Agent) {
                            Agent a = (Agent) instance;

							if(a != null)
							{
								
								if(process.getLaunchMode() != 0)//launch
								{
									Process p = a.getProcess();									
									p.kill(0);
								}
								else
								{//attached
									Process p = a.getProcess();
		                            if (a.isAttached()) {
		                                a.detach();
		                            }
									
								}
							}
							
                        }
                    } catch (Exception exc) {
                        exc.printStackTrace();
                    }
                }
            }
        }
    }

    public static void doAttach(Agent agent, TRCAgentProxy agentProxy) throws InactiveAgentException, InactiveProcessException {
        PIProcessListener processListener = (PIProcessListener) LoadersUtils.locateAgentListener(agentProxy);

        if (processListener == null) {
            processListener = new PIAttachListener(agentProxy.getProcessProxy().getNode(), agentProxy.getProcessProxy());
            LoadersUtils.registerAgentListener(agentProxy, processListener);
        }

        //process agent active command for this listener 
        processListener.monitor(true);
        processListener.setLaunchMode(0); //attach
        agent.addAgentListener(processListener);

        agent.attach();
    }

    public static final boolean launchTrace(TraceArguments trace, ArrayList filters, Vector options, String selectedProject, String selectedMonitor, ILaunch launch) {
        IPreferenceStore store = UIPlugin.getDefault().getPreferenceStore();
    	switchPerspective();

        try {
            String nodeName = trace.getHostName();
            Node lhost = profileConnect(nodeName, String.valueOf(trace.getPortNumber()));

            if (lhost == null) {
                return false;
            }

            nodeName = lhost.getName();

            //create the container
            IContainer container = createContainer(new Path(selectedProject));

            //create the monitor
            TRCMonitor monitor = createMonitor(container, selectedMonitor);

            //create the node
            TRCNode node = createNode(monitor, nodeName, String.valueOf(trace.getPortNumber()));

            PIProcessListener listener = new PIProcessListener(node);

            listener.monitor(true);

            Process lprocess = ProcessFactory.createProcess(lhost);

            //add process location
            if (ConnectUtil.isLocalHost(nodeName))
				lprocess.setLocation(trace.getLocation());
            else 
				lprocess.setLocation(null);
            
            lprocess.addProcessListener(listener);
            listener.setLaunchMode(1);
            listener.setAutoMonitoring(trace.isAutoMonitoring());

            boolean hasProfAgents = false;

            if (store.getBoolean(TraceConstants.LOG_OPTION_KEY)) {
                String agents = store.getString(TraceConstants.LOG_KEY);

                if ((agents != null) && !agents.equals("")) {
                    int idx = 0;

                    while (idx != -1) {
                        idx = agents.indexOf(",");

                        if (idx != -1) {
                            String agent = agents.substring(0, idx);
                            Agent a = AgentFactory.createAgent(lprocess, agent, TraceConstants.LOG_AGENT_TYPE);

                            a.addAgentListener(listener);

                            a.setAutoAttach(true);
                            agents = agents.substring(idx + 1);
                        } else {
                            Agent a = AgentFactory.createAgent(lprocess, agents, TraceConstants.LOG_AGENT_TYPE);

                            a.addAgentListener(listener);
                            a.setAutoAttach(true);
                        }
                    }
                }
            }

            if (store.getBoolean(TraceConstants.PROF_OPTION_KEY)) {
                hasProfAgents = true;

                Agent a = AgentFactory.createAgent(lprocess, TraceConstants.PROFILE_AGENT_NAME, TraceConstants.PROFILE_AGENT_TYPE);

                a.addAgentListener(listener);

                a.setAutoAttach(true);

                AgentConfiguration ac = a.getConfiguration();

                if (ac == null) {
                    a.setConfiguration(new AgentConfiguration());
                }

                // EC: make sure agent configuraiton is clear.	
                a.getConfiguration().clear();

                if (trace.isToProfileFile()) {
                    a.setProfileFile(trace.getProfileFile());
                }

                addDefaultProfileData(a, filters);
                addDefaultOptionsData(a, options);
            }

            String exec = trace.getClassName();

            if ((trace.getVMArguments() != null) && !trace.getVMArguments().equals("")) {
                exec = trace.getVMArguments() + " " + exec;
            }

            if ((trace.getParameters() != null) && !trace.getParameters().equals("")) {
                exec += (" " + trace.getParameters());
            }

            String param = "";

            if (!hasProfAgents) {
                param = exec;
            }

            if (hasProfAgents) {
                lprocess.setExecutable("java.exe");
                param = "-XrunpiAgent:server=controlled " + exec;
            } else {
                lprocess.setExecutable("java.exe");
                param = exec;
            }

            lprocess.setParameters(param);
            lprocess.addEnvironmentVariable(new Variable("CLASSPATH", TString.stripNewLineChar(trace.getClassPath())));
            lprocess.addEnvironmentVariable(new Variable("UICLASSPATH", trace.getClassPath()));
            listener.getProcessVariableList().add(new Variable("UICLASSPATH", trace.getClassPath()));

            setEnvironmentVariable(lprocess, trace.getEnvironmentVariable(), listener);
            
            try {
               lprocess.launch();
               
               final IProcess adapter = new ProcessAdapter(lprocess, launch, true, 1);
               TraceUIManager.getTraceUIManager().registerLaunchProcess(adapter);
               lprocess.addProcessListener(new ProcessListener() {
                  public void processLaunched(Process p) {
               	  }
               	  public void processExited(Process p) {
                     TraceUIManager.getTraceUIManager().deregisterLaunchProcess(adapter);
               		 DebugPlugin.getDefault().fireDebugEventSet(new DebugEvent[] {
                        new DebugEvent(adapter, DebugEvent.TERMINATE)
                     });
                  }
               });
               
               // add the mapping between TRCProcessProxy -> IProcess
               int pID = Integer.parseInt(lprocess.getProcessId());
               Iterator iter = node.getProcessProxies().iterator();
               while (iter.hasNext()) {
                  TRCProcessProxy proxy = (TRCProcessProxy)iter.next();
                  if (pID == proxy.getPid() && (proxy.isActive()==true)) { /** bug 83130 **/	
                     ProcessMap.put(proxy, adapter);
                     break;
                  }
               }
            } catch (Exception e) {
                String text = UIPlugin.getResourceString("PROCESS_LAUNCH_ERRTEXT_INFO_");
                final String msg = TString.change(UIPlugin.getResourceString("PROCESS_LAUNCH_ERROR_"), "%1", trace.getClassName());
                final Status err = new Status(Status.WARNING, ResourcesPlugin.PI_RESOURCES, IResourceStatus.INTERNAL_ERROR, text, null);

                HyadesUIPlugin.logError(e);
                
                Display.getDefault().syncExec(new Runnable() {
					public void run() {
		                ErrorDialog.openError(UIPlugin.getActiveWorkbenchShell(), UIPlugin.getResourceString("TRACE_MSG"), msg, err);
					}
                });
                
                return false;
            }
        } catch (Exception exc) {
            exc.printStackTrace();
            return false;
        }

        return true;
    }

    public static void setAgentConfiguration(TRCAgentProxy agent, Agent a) {
        AgentConfiguration aconfig = a.getConfiguration();

        aconfig.clear();

        //set profiling options
        if (agent.getType().equals(TraceConstants.PROFILE_AGENT_TYPE)) {
            Object[] configs = agent.getConfigurations().toArray();

            if (configs.length > 0) {
                TRCConfiguration config = (TRCConfiguration) configs[0];

                Vector v = new Vector();
                Object[] filters = config.getFilters().toArray();

                for (int idx = 0; idx < filters.length; idx++) {
                    TRCFilter filter = (TRCFilter) filters[idx];

                    if (filter.getActive().booleanValue()) {
                        AgentConfigurationEntry entry = new AgentConfigurationEntry();

                        /* RKD:  If the method portion of the filter is an empty String then this is a class filter */
                        if (filter.getOperation().equals("") || filter.getOperation().equals("\"\"")) {
                            entry.setName(filter.getPattern());
                            entry.setType("SETFILTER");
                            entry.setValue(filter.getMode());
                            entry.setEnabled(filter.getActive().booleanValue());
                        } else {
                            /* Method Filter */
                            entry.setName(filter.getPattern() + ' ' + filter.getOperation());
                            entry.setType("SETMETHODFILTER");
                            entry.setValue(filter.getMode());
                            entry.setEnabled(filter.getActive().booleanValue());
                        }

                        aconfig.addEntry(entry);
                    }
                }

                Object[] options = config.getOptions().toArray();

                for (int idx = 0; idx < options.length; idx++) {
                    TRCOption option = (TRCOption) options[idx];

                    AgentConfigurationEntry entry = new AgentConfigurationEntry();

                    entry.setType("SETOPTION");
                    entry.setName(option.getKey());
                    entry.setValue(option.getValue());

                    aconfig.addEntry(entry);
                }
            }
        }
    }

    /*
     * get the folder structure of a fully qualify file name
     */
    public static String getDirectoryName(String fName) {
        int i = fName.lastIndexOf(File.separatorChar);

        if (i >= 0) {
            fName = fName.substring(0, i);
        }

        return fName;
    }

    public static Enumeration getNodeConfiguration(String nodeName, int portNumber, boolean showErrorMsgs) {
        try {
            Node node = profileConnect(nodeName, String.valueOf(portNumber), showErrorMsgs);

            if (node == null) {
                return null;
            }

            return node.listProcesses();
        } catch (NotConnectedException exc2) {
            exc2.printStackTrace();
        }

        return null;
    }
    
    public static Enumeration getNodeConfiguration(String nodeName, int portNumber) {
    	return getNodeConfiguration(nodeName, portNumber, true);
    }    

    /*
     * check the folders existence of a fully qualify file path
     */
    public static boolean isPathExists(String fdir) {
        File pf = new File(getDirectoryName(fdir));

        return pf.exists();
    }

    public static boolean isZipFile(InputStream readStream) throws IOException {
        byte[] magic = new byte[2];

        if (readStream.read(magic) > 1) {
            if ((magic[0] == 'P') && (magic[1] == 'K')) {
                return true;
            }
        }

        return false;
    }

    public static boolean isZipFile(String fileName) throws IOException {
        FileInputStream readStream = new FileInputStream(fileName);
        boolean ret = isZipFile(readStream);

        readStream.close();
        readStream = null;

        return ret;
    }

    public static boolean createDirectoryMessage(String fDir, Shell shell) {
        File directory = new File(PDCoreUtil.getDirectoryName(fDir));
        String msg = UIPlugin.getResourceString("STR_FOLDER_CREATION_MSG");

        msg = TString.change(msg, "%1", directory.getPath());

        if (!MessageDialog.openQuestion(shell, UIPlugin.getResourceString("TRACE_MSG"), msg)) {
            return false;
        }

        if (!directory.mkdirs()) {
            String errMsg = UIPlugin.getResourceString("STR_ERROR_FOLDER_CREATION");

            MessageDialog.openError(shell, UIPlugin.getResourceString("TRACE_MSG"), errMsg);

            return false;
        }

        return true;
    }

    public static TRCAgentProxy createUniqueAgent(TRCProcessProxy processProxy, Agent agent) {
        long timeStamp = (new Date()).getTime();
        double time = ((double) timeStamp) / 1000;
        TRCAgentProxy agentProxy = createAgentNoCheck(processProxy, agent, timeStamp + "");
		LoadersUtils.registerAgentInstance(agentProxy,agent);
        LoadersUtils.setSessionStartTime(time, agentProxy);

        return agentProxy;
    }

    public static Node profileConnect(String host, String port, boolean showErrorMsgs) {
        String text = "";

        if (((host == null) || (host.trim() == "") || (port == null) || (port.trim() == "")) && showErrorMsgs) {
            text = UIPlugin.getResourceString("INVALID_HOST");

            final Status err = new Status(Status.WARNING, ResourcesPlugin.PI_RESOURCES, IResourceStatus.WARNING, text, null);

            Display.getDefault().syncExec(new Runnable() {
            	public void run() {
            		ErrorDialog.openError(UIPlugin.getActiveWorkbenchShell(), UIPlugin.getResourceString("TRACE_MSG"), UIPlugin.getResourceString("CONNECTION_FAIL_TEXT"), err);
            	}
            });

            return null;
        }

        IPreferenceStore store = UIPlugin.getDefault().getPreferenceStore();
        ConnectUtil util = new ConnectUtil(host, port, store.getString(HyadesConstants.SECURE_USERID), UIPlugin.getDefault());
        int result = util.connect(showErrorMsgs);

        if (result == ConnectUtil.CONNECTION_SUCCESS) { //store the user id
            store.setValue(HyadesConstants.SECURE_USERID, util.getUserId());

            return util.getNode();
        }

        return null;
    }
    
    public static Node profileConnect(String host, String port) {
    	return profileConnect(host, port, true);
    }    

    protected static TRCAgentProxy createAgentNoCheck(TRCProcessProxy process, Agent a, String sessionId) {
        TRCAgentProxy agent;
        String timestamp = DateFormat.getDateTimeInstance(DateFormat.FULL, DateFormat.FULL).format(new Date());
        String rID = timestamp;

        rID = TString.change(rID, " ", "");
        rID = TString.change(rID, ":", "");

        String agentName = new StringBuffer(process.getNode().getMonitor().getName()).append("_").append(process.getNode().getName()).append("_").append(process.getPid()).append("_").append(rID).append("_").append(sessionId).append("_").append(a.getName()).toString();

        agentName = TString.change(agentName, " ", "");

        String pPath = process.eResource().getURI().toString();
        IPath path = new Path(pPath);

        if (path.segmentCount() > 1) {
            pPath = path.removeLastSegments(1).toString();
        }

        String fileName = agentName + "." + TraceConstants.AGENT_EXT;
        IPath filePath = new Path(pPath).append(fileName);

        URI uri = URI.createURI(filePath.toString());

        Resource agDoc = Resource.Factory.Registry.INSTANCE.getFactory(uri).createResource(uri);
        EList agExt = agDoc.getContents();

        UIPlugin.getDefault().getResourceSet().getResources().add(agDoc); // prevents reloading later

        HierarchyFactory factory = UIPlugin.getDefault().getPerftraceFactory();

        agent = factory.createTRCAgentProxy();
        agent.setName(a.getName());

        if (a.getType() != null) {
            agent.setType(a.getType());
        }

        agent.setProcessProxy(process);

        TRCAgent iAgent = factory.createTRCAgent();

        iAgent.setAgentProxy(agent);
        agExt.add(iAgent);

		markModifiedResources(iAgent,true);

//        // adding support for multiple files
//        SaveUtil.addDocument(agDoc);

        //create configuration
        if (agent.getType().equals(TraceConstants.PROFILE_AGENT_TYPE)) {
            if (agent.getName().equals(TraceConstants.J2EE_AGENT_NAME)) {
                agent.setCollectionMode(TRCCollectionMode.HEAP_AND_EXECUTION_FULL_LITERAL);
            } else {
                TRCConfiguration config = factory.createTRCConfiguration();

                config.setName(getFilterSetName(a));   
                config.setActive(true);
                config.setAgentProxy(agent);
                agent.getConfigurations().add(config);

                AgentConfiguration aconfig = a.getConfiguration();
                boolean collectHeapInfo = false;
                boolean collectHeapInstanceLevelInfo = false;
                boolean collectExecInfo = false;
                boolean collectExecInstanceLevelInfo = false;
                boolean showExecutionFlowFull = false;

                if (aconfig != null) {
                    for (int idx = 0; idx < aconfig.size(); idx++) {
                        AgentConfigurationEntry entry = aconfig.getEntryAt(idx);

                        if(entry.getName().equals(IProfileLaunchConfigurationConstants.ATTR_SHOW_EXECUTION_FLOW))
                        {
                            TRCOption opt = factory.createTRCOption();
                            opt.setKey(entry.getName());
                            opt.setValue(entry.getValue());
                            opt.setConfiguration(config);
                        	showExecutionFlowFull = entry.getValue().equalsIgnoreCase("true");
                        }
                        else if(entry.getName().equals(IProfileLaunchConfigurationConstants.ATTR_SHOW_EXEC_INSTANCES))
                        {
                        	collectExecInstanceLevelInfo = entry.getValue().equalsIgnoreCase("true");
                        }
                        else if(entry.getName().equals(IProfileLaunchConfigurationConstants.ATTR_SHOW_HEAP_INSTANCES))
                        {
                        	collectHeapInstanceLevelInfo = entry.getValue().equalsIgnoreCase("true");
                        }
                        else if (entry.getType().equals("SETFILTER")) {
                            TRCFilter filter = factory.createTRCFilter();

                            filter.setMode(entry.getValue());
                            filter.setOperation("");
                            filter.setPattern(entry.getName());
                            filter.setType(entry.getValue());
                            filter.setActive(new Boolean(entry.isEnabled()));

                            filter.setConfiguration(config);
                        } else if (entry.getType().equals("SETMETHODFILTER")) {
                            String criteria = entry.getName();
                            int offset = criteria.indexOf(' ');

                            if (offset > 0) {
                                TRCFilter filter = factory.createTRCFilter();

                                filter.setMode(entry.getValue());
                                filter.setPattern(criteria.substring(0, offset));
                                filter.setOperation(criteria.substring(offset + 1, criteria.length()));
                                filter.setType(entry.getValue());
                                filter.setActive(new Boolean(entry.isEnabled()));
                                filter.setConfiguration(config);
                            }
                        } else if (entry.getType().equals("SETOPTION")) {
                            TRCOption opt = factory.createTRCOption();

                            opt.setKey(entry.getName());
                            opt.setValue(entry.getValue());
                            opt.setConfiguration(config);

                            if (entry.getName().equals(Options.OPTION_TRACE_MODE)) {
                                collectHeapInfo = !entry.getValue().equals(Options.OPTION_VALUE_NONE) && !entry.getValue().equals(Options.OPTION_VALUE_NOOBJECTCORRELATION);
                            }

                            if (entry.getName().equals(Options.OPTION_STACK_INFORMATION)) {
                                collectExecInfo = !entry.getValue().equals(Options.OPTION_VALUE_NONE);
                            }
                        }
                    }
                }

                
                /**
                 * 
                 * UI selections:
                 * a = collect heap info
                 * b = instance info in heap
                 * c = collect execution info
                 * d = exection full
                 * e = instance info in exection
                 * 
                 * ---------------------------
                 * |     | a | b | c | d | e |
                 * |-------------------------|   
                 * |HS   | 1 | 0 | 0 | - | - | HS = HEAP_STATISTICS_ONLY
                 * |-------------------------|   
                 * |HF   | 1 | 1 | 0 | - | - | HF = HEAP_FULL
                 * |-------------------------|   
                 * |HES  | 1 | 0 | 1 | 0 | - | HES = HEAP_AND_EXECUTION_STATISTICS_ONLY
                 * |-------------------------|   
                 * |HEF  | 1 | 0 | 1 | 1 | 0 | HEF = HEAP_AND_EXECUTION_FULL
                 * |-------------------------|   
                 * |HEF  | 1 | 0 | 1 | 1 | 1 | HEF = HEAP_AND_EXECUTION_FULL
                 * |-------------------------|   
                 * |HFES | 1 | 1 | 1 | 0 | - | HFES = HEAP_FULL_AND_EXECUTION_STATISTICS_ONLY
                 * |-------------------------|   
                 * |HEF  | 1 | 1 | 1 | 1 | 0 | HEF = HEAP_AND_EXECUTION_FULL
                 * |-------------------------|   
                 * |HEF  | 1 | 1 | 1 | 1 | 1 | HEF = HEAP_AND_EXECUTION_FULL
                 * |-------------------------|   
                 * |ES   | 0 | - | 1 | 0 | - | ES = EXECUTION_STATISTICS_ONLY
                 * |-------------------------|   
                 * |ENI  | 0 | - | 1 | 1 | 0 | ENI = EXECUTION_NO_INSTANCES
                 * |-------------------------|   
                 * |EF   | 0 | - | 1 | 1 | 1 | EF = EXECUTION_FULL
                 * |-------------------------| 
                 * 
                 */
                if (collectExecInfo) {
                    if (collectHeapInfo) // heap + execution
                    {
                    	if (collectHeapInstanceLevelInfo) // collect heap instance
                    	{
                    		if (showExecutionFlowFull) // exec full, no need to worry about instance level up to this level
                    			agent.setCollectionMode(TRCCollectionMode.HEAP_AND_EXECUTION_FULL_LITERAL);
                    		else // exec compressed, no instance collection available
                    			agent.setCollectionMode(TRCCollectionMode.HEAP_FULL_AND_EXECUTION_STATISTICS_ONLY_LITERAL);
                    	}
                    	else // no heap instance
                    	{
                    		if (showExecutionFlowFull) // exectution full, no need to worry about instance up to this level
                    			agent.setCollectionMode(TRCCollectionMode.HEAP_AND_EXECUTION_FULL_LITERAL);
                    		else // execution compress, no instance collection available
                    			agent.setCollectionMode(TRCCollectionMode.HEAP_AND_EXECUTION_STATISTICS_ONLY_LITERAL);
                    	}
                    } 
                    else // execution only
                    {
                        if (!showExecutionFlowFull) // exec compressed
                        {
                            agent.setCollectionMode(TRCCollectionMode.EXECUTION_STATISTICS_ONLY_LITERAL);
                        } 
                        else 
                        { // exec full
                        	if (collectExecInstanceLevelInfo) // collect exec instance
                        		agent.setCollectionMode(TRCCollectionMode.EXECUTION_FULL_LITERAL);
                        	else // no exec instance
                        		agent.setCollectionMode(TRCCollectionMode.EXECUTION_NO_INSTANCES_LITERAL);
                        }
                    }
                } 
                else // heap only
                {                	
                	if(collectHeapInstanceLevelInfo)// collect heap instance                        	                	
                        agent.setCollectionMode(TRCCollectionMode.HEAP_FULL_LITERAL);
                	else // no heap instance
                        agent.setCollectionMode(TRCCollectionMode.HEAP_STATISTICS_ONLY_LITERAL);
                }
            }
        }

        if (agent.getAgent() != null) {
            agent.getAgent().setCollectionMode(agent.getCollectionMode());
        }

        iAgent.setName(agent.getName());
        iAgent.setRuntimeId(agent.getRuntimeId());
        iAgent.setType(agent.getType());
        iAgent.setCollectionMode(agent.getCollectionMode());
 //       System.out.println("Collection Mode="+agent.getCollectionMode());

        return agent;
    }

    /**
     * Sets the modified flag on the Hierarchy resources up to the monitor.
     * 
	 */
	public static void markModifiedResources(EObject eObject, boolean modifiedState) {
		while(true)
		{
			if(eObject==null)
				break;
			if(eObject instanceof TRCAgent)
			{
				eObject.eResource().setModified(modifiedState);
				eObject = ((TRCAgent)eObject).getAgentProxy().getProcessProxy();
				continue;
			}
			if(eObject instanceof TRCAgentProxy)
			{
				eObject.eResource().setModified(modifiedState);
				eObject = ((TRCAgentProxy)eObject).getProcessProxy().getNode();
				continue;
			}
			if(eObject instanceof TRCProcessProxy)
			{
				eObject.eResource().setModified(modifiedState);
				eObject = ((TRCProcessProxy)eObject).getNode();
				continue;
			}
			if(eObject instanceof TRCNode)
			{
				eObject.eResource().setModified(modifiedState);
				eObject = ((TRCNode)eObject).getMonitor();
				continue;
			}
			if(eObject instanceof TRCMonitor)
			{
				eObject.eResource().setModified(modifiedState);
			}
			break;
		}
	}

    private static void addDefaultOptionsData(Agent a, Vector optionsList) {
        AgentConfiguration config = a.getConfiguration();

        for (int idx = 0; idx < optionsList.size(); idx++) {
            config.addEntry((AgentConfigurationEntry) optionsList.elementAt(idx));
        }
    }

    private static void addDefaultProfileData(Agent a, ArrayList filtersList) {
        AgentConfiguration config = a.getConfiguration();

        for (int idx = 0; idx < filtersList.size(); idx++) {
            FilterTableElement elem = (FilterTableElement) filtersList.get(idx);

            AgentConfigurationEntry entry = new AgentConfigurationEntry();

            /* RKD:  If there is no method filtercriteria then just send the class info */
            if (elem.getMethod().equals("")) {
                entry.setName(elem.getText());
                entry.setType("SETFILTER");
            } else {
                /* RKD:  The method filter criteria is specified by placing a space between the two strings */
                entry.setName(elem.getText() + ' ' + elem.getMethod());
                entry.setType("SETMETHODFILTER");
            }

            entry.setValue(elem.getVisibility().equals(UIPlugin.getResourceString("STR_INCLUDE"))?"INCLUDE":"EXCLUDE");
            entry.setEnabled(true);

            config.addEntry(entry);
        }
    }

    private static void setAgent(TRCAgentProxy agent, Process process, PIProcessListener listener) {
        Agent a = AgentFactory.createAgent(process, agent.getName(), agent.getType());

        a.addAgentListener(listener);

        setAgentConfiguration(agent, a);
    }

    private static void setEnvironmentVariable(Process process, String[] envVariables, PIProcessListener listener) {
        if (envVariables != null && process != null) {
	        for (int i = 0; i < envVariables.length; i++) {
	            try {
	            	StringTokenizer tok = new StringTokenizer(envVariables[i], "=");
	            	String name = tok.nextToken();
	            	String value = tok.nextToken();
	                process.addEnvironmentVariable(new Variable(name, value));
	                listener.getProcessVariableList().add(new Variable(name, value));
	            }
	            catch (Exception e) {
	            	HyadesUIPlugin.logError(e);
	            }
	        }
        }
    }

	public static void copyAgentProperties(TRCAgentProxy fAgentOld, TRCAgentProxy fAgent) {
		fAgent.setCollectionMode(fAgentOld.getCollectionMode());
		fAgent.setProfileFile(fAgentOld.getProfileFile());
		fAgent.getConfigurations().clear();
		for (Iterator iter = fAgentOld.getConfigurations().iterator(); iter.hasNext();) {
			TRCConfiguration configuration = (TRCConfiguration) iter.next();
			fAgent.getConfigurations().add(EcoreUtil.copy(configuration));
			
		};
		TRCProcessProxy process = fAgent.getProcessProxy();
		TRCNode node = process.getNode();
		sendConfigurationToAgent(fAgent, node.getName(), process.getPid());			
				
	}
	private static void sendConfigurationToAgent(TRCAgentProxy agent, String host, int processId)
	{
		Node node = PDCoreUtil.profileConnect(host, String.valueOf(agent.getProcessProxy().getNode().getPort()));
		if(node == null)
			return;

	
		Process p = node.getProcess(String.valueOf(processId));

		if(p != null)
		{
			try {
				Agent a = p.getAgent(agent.getName());
				a.setProfileFile(agent.getProfileFile());
				PDCoreUtil.setAgentConfiguration(agent, a);
				if(a != null && a.isActive()) {
				  a.publishConfiguration();
			  
				  /* Apply the filters */
				  CustomCommand command = new CustomCommand();
				  command.setData("APPLYFILTERS");
				  a.invokeCustomCommand(command);			  
				}			
			}
			catch(InactiveAgentException aex)
			{
			}
		}
	}
	
	private static void switchPerspective() {
        Display.getDefault().syncExec(new Runnable() {
        	public void run() {
        		try{
    		 		IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
    		 		boolean shouldSwitch = HyadesUIPlugin.getInstance().openPerspectiveSwitchDialog(
    						window.getShell(), PDPerspective.ID_TRACE_PERSPECTIVE,
    						UIPlugin.getDefault().getPreferenceStore(), 
    						HyadesUI.SWITCH_TO_PERSPECTIVE_KEY, 
    						UIPlugin.getResourceString("CONFIRM_PERSP_SWITCH_MSG"));
    			  	if(shouldSwitch) {
    			  		UIPlugin.getActivePage().showView(PDPerspective.ID_PD_NAVIGATOR_VIEW);
    			  	}
    		    	if (PDPerspective.ID_TRACE_PERSPECTIVE.equals(window.getActivePage().getPerspective().getId())) {
   						PDProjectExplorer nav = (PDProjectExplorer)UIPlugin.getActivePage().showView(PDPerspective.ID_PD_NAVIGATOR_VIEW);
   						nav.getViewer().refresh();
    		        }
        		} catch (Exception exc) {
        			HyadesUIPlugin.logError(exc);
        		}
        	}
        });

	}
	
	/**
	 * Retrieves the filter set name for the given agent.
	 */
	private static String getFilterSetName(Agent a) {
        AgentConfiguration conf = a.getConfiguration();
        for (int i=0;i<conf.size();++i) {
        	AgentConfigurationEntry e = conf.getEntryAt(i);
        	if (IProfileLaunchConfigurationConstants.ATTR_FILTER_SET.equals(e.getName())) {
        		return e.getValue();
        	}
        }
        return UIPlugin.getResourceString(TraceConstants.DEFAULT_FILTER_SET);
	}
}
